#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace HeuristicLab.Core {
  /// <summary>
  /// The visual representation of the information of the variables of an operator.
  /// </summary>
  public partial class OperatorBaseParametersView : ViewBase {
    /// <summary>
    /// Gets or sets the operator whose variable infos should be represented visually.
    /// </summary>
    /// <remarks>Uses property <see cref="ViewBase.Item"/> of base class <see cref="ViewBase"/>.
    /// No own data storage present.</remarks>
    public IOperator Operator {
      get { return (IOperator)Item; }
      set { base.Item = value; }
    }
    /// <summary>
    /// Gets all selected variable infos.
    /// <note type="caution"> Variable infos are returned read-only!</note>
    /// </summary>
    public ICollection<IParameter> SelectedParameter {
      get {
        List<IParameter> selected = new List<IParameter>();
        foreach (ListViewItem item in parametersListView.SelectedItems)
          selected.Add((IParameter)item.Tag);
        return selected.AsReadOnly();
      }
    }

    /// <summary>
    /// Initializes a new instance of <see cref="OperatorBaseVariableInfosView"/> with caption "Operator".
    /// </summary>
    public OperatorBaseParametersView() {
      InitializeComponent();
      parametersListView.Columns[0].Width = Math.Max(0, parametersListView.Width - 25);
      Caption = "Operator";
    }
    /// <summary>
    /// Initializes a new instance of <see cref="OperatorBaseVariableInfosView"/> with the given operator
    /// <paramref name="op"/>.
    /// </summary>
    /// <remarks>Calls <see cref="OperatorBaseVariableInfosView()"/>.</remarks>
    /// <param name="op">The operator whose variable infos should be displayed.</param>
    public OperatorBaseParametersView(IOperator op)
      : this() {
      Operator = op;
    }

    /// <summary>
    /// Removes the eventhandlers from the underlying <see cref="IOperator"/>.
    /// </summary>
    /// <remarks>Calls <see cref="ViewBase.RemoveItemEvents"/> of base class <see cref="ViewBase"/>.</remarks>
    protected override void RemoveItemEvents() {
      Operator.ParameterAdded -= new EventHandler<ParameterEventArgs>(OperatorBase_ParameterAdded);
      Operator.ParameterRemoved -= new EventHandler<ParameterEventArgs>(OperatorBase_ParameterRemoved);
      base.RemoveItemEvents();
    }
    /// <summary>
    /// Adds eventhandlers to the underlying <see cref="IOperator"/>.
    /// </summary>
    /// <remarks>Calls <see cref="ViewBase.AddItemEvents"/> of base class <see cref="ViewBase"/>.</remarks>
    protected override void AddItemEvents() {
      base.AddItemEvents();
      Operator.ParameterAdded += new EventHandler<ParameterEventArgs>(OperatorBase_ParameterAdded);
      Operator.ParameterRemoved += new EventHandler<ParameterEventArgs>(OperatorBase_ParameterRemoved);
    }

    /// <summary>
    /// Updates all controls with the latest data of the model.
    /// </summary>
    /// <remarks>Calls <see cref="ViewBase.UpdateControls"/> of base class <see cref="ViewBase"/>.</remarks>
    protected override void UpdateControls() {
      base.UpdateControls();
      parameterDetailsGroupBox.Controls.Clear();
      if (Operator == null) {
        Caption = "Operator";
        parametersListView.Enabled = false;
        parameterDetailsGroupBox.Enabled = false;
      } else {
        Caption = Operator.Name + " (" + Operator.GetType().Name + ")";
        parametersListView.Enabled = true;
        parametersListView.Items.Clear();
        foreach (IParameter parameter in Operator.Parameters) {
          ListViewItem item = new ListViewItem();
          item.Text = parameter.Name;
          item.Tag = parameter;
          parametersListView.Items.Add(item);
        }
        if (parametersListView.Items.Count > 0)
          parametersListView.SelectedIndices.Add(0);
      }
    }

    private void parametersListView_SelectedIndexChanged(object sender, EventArgs e) {
      if (parameterDetailsGroupBox.Controls.Count > 0)
        parameterDetailsGroupBox.Controls[0].Dispose();
      parameterDetailsGroupBox.Controls.Clear();
      parameterDetailsGroupBox.Enabled = false;
      if (parametersListView.SelectedItems.Count == 1) {
        IParameter parameter = (IParameter)parametersListView.SelectedItems[0].Tag;
        Control control = (Control)parameter.CreateView();
        parameterDetailsGroupBox.Controls.Add(control);
        control.Dock = DockStyle.Fill;
        parameterDetailsGroupBox.Enabled = true;
      }
      OnSelectedParametersChanged();
    }

    /// <summary>
    /// Occurs when the variables were changed, whose infos should be displayed.
    /// </summary>
    public event EventHandler SelectedParametersChanged;
    /// <summary>
    /// Fires a new <c>SelectedVariableInfosChanged</c>.
    /// </summary>
    protected virtual void OnSelectedParametersChanged() {
      if (SelectedParametersChanged != null)
        SelectedParametersChanged(this, new EventArgs());
    }

    #region Size Changed Events
    private void parametersListView_SizeChanged(object sender, EventArgs e) {
      if (parametersListView.Columns.Count > 0)
        parametersListView.Columns[0].Width = Math.Max(0, parametersListView.Width - 25);
    }
    #endregion

    #region OperatorBase Events
    private void OperatorBase_ParameterAdded(object sender, ParameterEventArgs e) {
      ListViewItem item = new ListViewItem();
      item.Text = e.Parameter.Name;
      item.Tag = e.Parameter;
      parametersListView.Items.Add(item);
    }
    private void OperatorBase_ParameterRemoved(object sender, ParameterEventArgs e) {
      ListViewItem itemToDelete = null;
      foreach (ListViewItem item in parametersListView.Items) {
        if (item.Tag == e.Parameter)
          itemToDelete = item;
      }
      parametersListView.Items.Remove(itemToDelete);
    }
    #endregion
  }
}
