#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace HeuristicLab.Core {
  /// <summary>
  /// Represents a parameter of an operator.
  /// </summary>
  /// <typeparam name="T">The data type of the parameter, which has to be an <see cref="IItem"/>.</typeparam>
  public class Parameter<T> : ItemBase, IParameter, IParamter<T> where T : class, IItem {
    private string myActualName;
    /// <summary>
    /// Gets or sets the actual name of the current instance.
    /// </summary>
    /// <remarks>Calls <see cref="OnActualNameChanged"/> in the setter.</remarks>
    public string ActualName {
      get { return myActualName; }
      set {
        if (! myActualName.Equals(value)) {
          myActualName = value;
          OnActualNameChanged();
        }
      }
    }
    private string myFormalName;
    /// <summary>
    /// Gets the formal name of the current instance.
    /// </summary>
    public string FormalName {
      get { return myFormalName; }
    }
    private string myDescription;
    /// <summary>
    /// Gets or sets the description of the current instance.
    /// </summary>
    /// <remarks>Calls <see cref="OnDescriptionChanged"/> in the setter.</remarks>
    public string Description {
      get { return myDescription; }
      set {
        if (! myDescription.Equals(value)) {
          myDescription = value;
          OnDescriptionChanged();
        }
      }
    }
    /// <summary>
    /// Gets the data type of the parameter.
    /// </summary>
    public Type DataType {
      get { return typeof(T); }
    }
    private ParameterType myType;
    /// <summary>
    /// Gets the type of the parameter (input, output, both).
    /// </summary>
    public ParameterType Type {
      get { return myType; }
    }
    private bool myConstant;
    /// <summary>
    /// Gets or sets, if the parameter is constant (only valid for input parameters).
    /// </summary>
    /// <remarks>Calls <see cref="OnConstantChanged"/> in the setter.</remarks>
    public bool Constant {
      get { return myConstant; }
      set {
        if (Type != ParameterType.In) throw new InvalidOperationException("only input parameters can be constant");
        if (myConstant != value) {
          myConstant = value;
          OnConstantChanged();
        }
      }
    }
    private bool myCached;
    /// <summary>
    /// Gets or sets, if the parameter should be cached by operators.
    /// </summary>
    /// <remarks>Calls <see cref="OnCachedChanged"/> in the setter.</remarks>
    public bool Cached {
      get { return myCached; }
      set {
        if (myCached != value) {
          myCached = value;
          OnCachedChanged();
        }
      }
    }
    private T myValue;
    /// <summary>
    /// Gets or sets the value of the parameter.
    /// </summary>
    public T Value {
      get { return myValue; }
      set {
        if (! myValue.Equals(value)) {
          myValue = value;
          OnValueChanged();
        }
      }
    }
    /// <summary>
    /// Gets or sets the value of the parameter.
    /// </summary>
    public IItem IParameter.Value {
      get { return myValue; }
      set { Value = (T)value; }
    }

    /// <summary>
    /// Initializes a new instance of <see cref="Parameter"/> with name "Anonymous",
    /// no description, and a <c>null</c> data type. The type of 
    /// the parameter is input.
    /// </summary>
    public Parameter() {
      myActualName = "Anonymous";
      myFormalName = "Anonymous";
      myDescription = "";
      myType = ParameterType.In;
      myConstant = false;
      myCached = false;
      myValue = null;
    }
    /// <summary>
    /// Initializes a new instance of <see cref="Parameter"/> with the given parameters.
    /// </summary>
    /// <remarks>Calls <see cref="Parameter()"/>.<br/>
    /// The formal name is assigned to the actual name, too.</remarks>
    /// <param name="formalName">The formal name of the current instance.</param>
    /// <param name="description">The description of the current instance.</param>
    /// <param name="kind">The type of the parameter.</param>
    public Parameter(string formalName, string description, ParameterType type, bool constant, bool cached)
      : this() {
      myActualName = formalName;
      myFormalName = formalName;
      myDescription = description;
      myType = type;
      myConstant = constant;
      myCached = cached;
    }

    /// <summary>
    /// Creates a new instance of <see cref="ParameterView"/> to represent the current instance
    /// visually.
    /// </summary>
    /// <returns>The created view as <see cref="ParameterView"/>.</returns>
    public override IView CreateView() {
      return new ParameterView(this);
    }

    /// <summary>
    /// Clones the current instance (deep clone).
    /// </summary>
    /// <param name="clonedObjects">Dictionary of all already cloned objects. (Needed to avoid cycles.)</param>
    /// <returns>The cloned object as <see cref="Parameter"/>.</returns>
    public override object Clone(IDictionary<Guid, object> clonedObjects) {
      Parameter<T> clone = (Parameter<T>)base.Clone(clonedObjects);
      clonedObjects.Add(Guid, clone);
      clone.myActualName = ActualName;
      clone.myFormalName = FormalName;
      clone.myDescription = Description;
      clone.myType = Type;
      clone.myConstant = Constant;
      clone.myCached = Cached;
      clone.myValue = (T)Auxiliary.Clone(Value, clonedObjects);
      return clone;
    }

    /// <inheritdoc/>
    public event EventHandler ActualNameChanged;
    /// <summary>
    /// Fires a new <c>ActualNameChanged</c> event.
    /// </summary>
    protected virtual void OnActualNameChanged() {
      if (ActualNameChanged != null)
        ActualNameChanged(this, new EventArgs());
    }
    /// <inheritdoc />
    public event EventHandler DescriptionChanged;
    /// <summary>
    /// Fires a new <c>DescriptionChanged</c> event.
    /// </summary>
    protected virtual void OnDescriptionChanged() {
      if (DescriptionChanged != null)
        DescriptionChanged(this, new EventArgs());
    }
    public event EventHandler ConstantChanged;
    /// <summary>
    /// Fires a new <c>ConstantChanged</c> event.
    /// </summary>
    protected virtual void OnConstantChanged() {
      if (ConstantChanged != null)
        ConstantChanged(this, new EventArgs());
    }
    public event EventHandler CachedChanged;
    /// <summary>
    /// Fires a new <c>CachedChanged</c> event.
    /// </summary>
    protected virtual void OnCachedChanged() {
      if (CachedChanged != null)
        CachedChanged(this, new EventArgs());
    }
    public event EventHandler ValueChanged;
    /// <summary>
    /// Fires a new <c>ValueChanged</c> event.
    /// </summary>
    protected virtual void OnValueChanged() {
      if (ValueChanged != null)
        ValueChanged(this, new EventArgs());
    }

    #region Persistence Methods
    /// <summary>
    /// Saves the current instance as <see cref="XmlNode"/> in the specified <paramref name="document"/>.
    /// </summary>
    /// <remarks>Calls <see cref="StorableBase.GetXmlNode"/> of base class <see cref="ItemBase"/>. <br/>
    /// A quick overview how the single elements of the current instance are saved:
    /// <list type="bullet">
    /// <item>
    /// <term>ActualName: </term>
    /// <description>Saved as <see cref="XmlAttribute"/> with tag name <c>ActualName</c>.</description>
    /// </item>
    /// <item>
    /// <term>FormalName: </term>
    /// <description>Saved as <see cref="XmlAttribute"/> with tag name <c>FormalName</c>.</description>
    /// </item>
    /// <item>
    /// <term>Description: </term>
    /// <description>Saved as <see cref="XmlAttribute"/> with tag name <c>Description</c>.</description>
    /// </item>
    /// <item>
    /// <term>Type: </term>
    /// <description>Saved as <see cref="XmlAttribute"/> with tag name <c>ParameterType</c>.</description>
    /// </item>
    /// <item>
    /// <term>Constant: </term>
    /// <description>Saved as <see cref="XmlAttribute"/> with tag name <c>Constant</c>.</description>
    /// </item>
    /// <item>
    /// <term>Cached: </term>
    /// <description>Saved as <see cref="XmlAttribute"/> with tag name <c>Cached</c>.</description>
    /// </item>
    /// <item>
    /// <term>Value: </term>
    /// <description>Saved as child node with tag name <c>Value</c>.</description>
    /// </item>
    /// </list></remarks>
    /// <param name="name">The (tag)name of the <see cref="XmlNode"/>.</param>
    /// <param name="document">The <see cref="XmlDocument"/> where to save the data.</param>
    /// <param name="persistedObjects">The dictionary of all already persisted objects. (Needed to avoid cycles.)</param>
    /// <returns>The saved <see cref="XmlNode"/>.</returns>
    public override XmlNode GetXmlNode(string name, XmlDocument document, IDictionary<Guid,IStorable> persistedObjects) {
      XmlNode node = base.GetXmlNode(name, document, persistedObjects);
      XmlAttribute actualNameAttribute = document.CreateAttribute("ActualName");
      actualNameAttribute.Value = ActualName;
      node.Attributes.Append(actualNameAttribute);

      XmlAttribute formalNameAttribute = document.CreateAttribute("FormalName");
      formalNameAttribute.Value = FormalName;
      node.Attributes.Append(formalNameAttribute);

      XmlAttribute descriptionAttribute = document.CreateAttribute("Description");
      descriptionAttribute.Value = Description;
      node.Attributes.Append(descriptionAttribute);

      XmlAttribute typeAttribute = document.CreateAttribute("ParameterType");
      typeAttribute.Value = Type.ToString();
      node.Attributes.Append(typeAttribute);

      XmlAttribute constantAttribute = document.CreateAttribute("Constant");
      constantAttribute.Value = Constant.ToString();
      node.Attributes.Append(constantAttribute);

      XmlAttribute cachedAttribute = document.CreateAttribute("Cached");
      cachedAttribute.Value = Cached.ToString();
      node.Attributes.Append(cachedAttribute);

      if (myValue != null)
        node.AppendChild(PersistenceManager.Persist("Value", Value, document, persistedObjects));

      return node;
    }
    /// <summary>
    /// Loads the persisted parameter from the specified <paramref name="node"/>.
    /// </summary>
    /// <remarks>See <see cref="GetXmlNode"/> for further information on how the parameter must be 
    /// saved. <br/>
    /// Calls <see cref="StorableBase.Populate"/> of base class <see cref="ItemBase"/>.</remarks>
    /// <param name="node">The <see cref="XmlNode"/> where the parameter is saved.</param>
    /// <param name="restoredObjects">The dictionary of all already restored objects. 
    /// (Needed to avoid cycles.)</param>
    public override void Populate(XmlNode node, IDictionary<Guid,IStorable> restoredObjects) {
      base.Populate(node, restoredObjects);
      myActualName = node.Attributes["ActualName"].Value;
      myFormalName = node.Attributes["FormalName"].Value;
      myDescription = node.Attributes["Description"].Value;
      myType = (ParameterType)Enum.Parse(typeof(ParameterType), node.Attributes["ParameterType"].Value);
      myConstant = bool.Parse(node.Attributes["Constant"].Value);
      myCached = bool.Parse(node.Attributes["Cached"].Value);

      XmlNode valueNode = node.SelectSingleNode("Value");
      if (valueNode != null)
        myValue = (T)PersistenceManager.Restore(valueNode, restoredObjects);
    }
    #endregion
  }
}
